@@ -4,6 +4,8 @@ from typing import TYPE_CHECKING, Any, List, Optional, TypedDict, Union, cast
 from ..event import Event, EventBase
 from .device_class import DeviceClass, DeviceClassDataType
 from .device_config import DeviceConfig, DeviceConfigDataType
+from .endpoint import Endpoint, EndpointDataType
+from .notification import Notification, NotificationDataType
 from .value import (
     MetaDataType,
     Value,
@@ -24,29 +26,30 @@ class NodeDataType(TypedDict, total=False):
     name: str
     location: str
     status: int  # 0-4  # required
-    deviceClass: DeviceClassDataType  # required
+    deviceClass: DeviceClassDataType
     zwavePlusVersion: int
     nodeType: int
     roleType: int
-    isListening: bool  # required
-    isFrequentListening: bool  # required
-    isRouting: bool  # required
-    maxBaudRate: int  # required
-    isSecure: bool  # required
-    isBeaming: bool  # required
-    version: int  # required
+    isListening: bool
+    isFrequentListening: bool
+    isRouting: bool
+    maxBaudRate: int
+    isSecure: bool
+    isBeaming: bool
+    version: int
     firmwareVersion: str
-    manufacturerId: int  # required
-    productId: int  # required
-    productType: int  # required
+    manufacturerId: int
+    productId: int
+    productType: int
     deviceConfig: DeviceConfigDataType
-    neighbors: List[int]  # required
-    keepAwake: bool  # required
-    index: int  # TODO: I can't the below items in the docs.
+    neighbors: List[int]
+    keepAwake: bool
+    index: int
     installerIcon: int
     userIcon: int
     ready: bool
     label: str
+    endpoints: List[EndpointDataType]
     endpointCountIsDynamic: bool
     endpointsHaveIdenticalCapabilities: bool
     individualEndpointCount: int
@@ -99,62 +102,62 @@ class Node(EventBase):
     @property
     def ready(self) -> Optional[bool]:
         """Return the ready."""
-        return self.data.get("ready")  # TODO: I can't find ready in the docs.
+        return self.data.get("ready")
 
     @property
     def device_class(self) -> DeviceClass:
         """Return the device_class."""
-        return DeviceClass(self.data["deviceClass"])
+        return DeviceClass(self.data.get("deviceClass", {}))
 
     @property
-    def is_listening(self) -> bool:
+    def is_listening(self) -> Optional[bool]:
         """Return the is_listening."""
-        return self.data["isListening"]
+        return self.data.get("isListening")
 
     @property
-    def is_frequent_listening(self) -> bool:
+    def is_frequent_listening(self) -> Optional[bool]:
         """Return the is_frequent_listening."""
         return self.data["isFrequentListening"]
 
     @property
-    def is_routing(self) -> bool:
+    def is_routing(self) -> Optional[bool]:
         """Return the is_routing."""
-        return self.data["isRouting"]
+        return self.data.get("isRouting")
 
     @property
-    def max_baud_rate(self) -> int:
+    def max_baud_rate(self) -> Optional[int]:
         """Return the max_baud_rate."""
-        return self.data["maxBaudRate"]
+        return self.data.get("maxBaudRate")
 
     @property
-    def is_secure(self) -> bool:
+    def is_secure(self) -> Optional[bool]:
         """Return the is_secure."""
-        return self.data["isSecure"]
+        return self.data.get("isSecure")
 
     @property
-    def version(self) -> int:
+    def version(self) -> Optional[int]:
         """Return the version."""
-        return self.data["version"]
+        return self.data.get("version")
 
     @property
-    def is_beaming(self) -> bool:
+    def is_beaming(self) -> Optional[bool]:
         """Return the is_beaming."""
-        return self.data["isBeaming"]
+        return self.data.get("isBeaming")
 
     @property
-    def manufacturer_id(self) -> int:
+    def manufacturer_id(self) -> Optional[int]:
         """Return the manufacturer_id."""
-        return self.data["manufacturerId"]
+        return self.data.get("manufacturerId")
 
     @property
-    def product_id(self) -> int:
+    def product_id(self) -> Optional[int]:
         """Return the product_id."""
-        return self.data["productId"]
+        return self.data.get("productId")
 
     @property
-    def product_type(self) -> int:
+    def product_type(self) -> Optional[int]:
         """Return the product_type."""
-        return self.data["productType"]
+        return self.data.get("productType")
 
     @property
     def firmware_version(self) -> Optional[str]:
@@ -199,7 +202,12 @@ class Node(EventBase):
     @property
     def neighbors(self) -> List[int]:
         """Return the neighbors."""
-        return self.data["neighbors"]
+        return self.data.get("neighbors", [])
+
+    @property
+    def endpoints(self) -> List[Endpoint]:
+        """Return the endpoints."""
+        return [Endpoint(endpoint) for endpoint in self.data["endpoints"]]
 
     @property
     def endpoint_count_is_dynamic(self) -> Optional[bool]:
@@ -294,6 +302,18 @@ class Node(EventBase):
             }
         )
 
+    async def async_poll_value(self, val: Union[Value, str]) -> Any:
+        """Send pollValue command to Node for given value (or value_id)."""
+        # a value may be specified as value_id or the value itself
+        if not isinstance(val, Value):
+            val = self.values[val]
+        # the value object needs to be send to the server
+        result = await self.client.async_send_command(
+            {"command": "node.poll_value", "nodeId": self.node_id, "valueId": val.data}
+        )
+        # result may or may not be there
+        return result.get("result")
+
     def handle_wake_up(self, event: Event) -> None:
         """Process a node wake up event."""
 
@@ -314,6 +334,16 @@ class Node(EventBase):
 
     def handle_ready(self, event: Event) -> None:
         """Process a node ready event."""
+        # the event contains a full dump of the node
+        self.data.update(event.data["nodeState"])
+        # update/add values
+        for value_state in event.data["nodeState"]["values"]:
+            value_id = get_value_id(self, value_state)
+            value = self.values.get(value_id)
+            if value is None:
+                self.values[value_id] = Value(self, value_state)
+            else:
+                value.update(value_state)
 
     def handle_value_added(self, event: Event) -> None:
         """Process a node value added event."""
@@ -324,19 +354,12 @@ class Node(EventBase):
         """Process a node value updated event."""
         value = self.values.get(get_value_id(self, event.data["args"]))
         if value is None:
-            # TODO decide how to handle value updated for unknown values
-            print()
-            print(
-                "Value updated for unknown value",
-                get_value_id(self, event.data["args"]),
-            )
-            print("Available value IDs", ", ".join(self.values))
-            print()
-            value = Value(self, event.data["args"])
-            self.values[value.value_id] = value
+            # received update for unknown value
+            # should not happen but just in case, treat like added value
+            self.handle_value_added(event)
         else:
             value.receive_event(event)
-        event.data["value"] = value
+            event.data["value"] = value
 
     def handle_value_removed(self, event: Event) -> None:
         """Process a node value removed event."""
@@ -344,13 +367,22 @@ class Node(EventBase):
 
     def handle_value_notification(self, event: Event) -> None:
         """Process a node value notification event."""
-        event.data["notification"] = ValueNotification.from_event(event)
+        # append metadata if value metadata is available
+        value = self.values.get(get_value_id(self, event.data["args"]))
+        if value and value.data.get("metadata"):
+            event.data["args"]["metadata"] = value.data["metadata"]
+        event.data["value_notification"] = ValueNotification(self, event.data["args"])
 
     def handle_metadata_updated(self, event: Event) -> None:
         """Process a node metadata updated event."""
+        # handle metadata updated as value updated (a its a value object with included metadata)
+        self.handle_value_updated(event)
 
     def handle_notification(self, event: Event) -> None:
         """Process a node notification event."""
+        event.data["notification"] = Notification(
+            self, cast(NotificationDataType, event.data)
+        )
 
     def handle_firmware_update_progress(self, event: Event) -> None:
         """Process a node firmware update progress event."""
